# -*- coding: utf-8 -*-
"""
蜂巢的排序set，可以用来存储顺序敏感的序列.
"""
import collections
from .base import HoneyBean

class SortedSet(HoneyBean, collections.MutableSet):
    def __init__(self, *args, **kwargs):
        super(SortedSet, self).__init__(*args, **kwargs)
        lua = """
        local ret_value = 0
        if not redis.call('ZSCORE', KEYS[1], ARGV[2]) then
          redis.call('ZADD', KEYS[1], ARGV[1], ARGV[2])
          ret_value = 1
        end
        return ret_value"""
        self.lua_addnx = self.redis.register_script(lua)

    def __len__(self):
        return int(self.redis.zcard(self.key))

    # replace zrank with zscore O(log(N)) -> O(1)
    # http://redis.io/commands/zscore
    def __contains__(self, elem):
        rank = self.redis.zscore(self.key, self._pickle(elem))
        return False if rank is None else True

    def remove(self, elem):
        removed_count = self.redis.zrem(self.key, self._pickle(elem))
        if not removed_count:
            raise KeyError('elem was not in the set')

    def discard(self, elem):
        return self.redis.zrem(self.key, self._pickle(elem))

    def _data(self, pipe=None, last=False, withscore=True):
        limit = 500
        offset = 0

        has_more = True
        while has_more:
            has_more = False
            count = 0
            for v in self._chunk(limit, offset, withscore, last, pipe):
                count += 1
                yield v

            has_more = count >= limit
            if has_more:
                offset += limit
            else:
                break

    def __iter__(self):
        return self._data(last=False)

    def __reversed__(self):
        return self._data(last=True)

    def chunk(self, limit, offset, last=False, withscore=True, from_score=None, to_score=None):
        return self._chunk(limit, offset, withscore, last=last, pipe=self.redis, from_score=from_score, to_score=to_score)

    def rchunk(self, limit, offset):
        return self.chunk(limit, offset, last=True)

    # 获取element 的score. 接收任意参数，返回有序list
    def score_by_element(self, elem):
        if not elem:
            return []

        if type(elem) not in (list, tuple):
            elem = [elem]
        print elem
        pip = self.redis
        return [pip.zscore(self.key, self._pickle(e)) for e in elem]

    def _chunk(self, limit, offset, withscore, last=False, pipe=None, from_score=None, to_score=None):
        redis = pipe if pipe is not None else self.redis
        if from_score and to_score:
            vals = redis.zrangebyscore(
                self.key,
                from_score,
                to_score,
                offset,
                offset + (limit - 1),
                withscores=withscore,
                score_cast_func=float
            )
        else:
            vals = redis.zrange(
                self.key,
                offset,
                offset + (limit - 1),
                desc=last,
                withscores=withscore,
                score_cast_func=float
            )
        if vals:
            for v in vals:
                if withscore:
                    yield self._unpickle(v[0]), v[1]
                else:
                    yield self._unpickle(v)

    def _update(self, data, pipe=None):
        super(SortedSet, self)._update(data, pipe)
        p = pipe if pipe is not None else self.redis.pipeline()

        for elem in data:
            self._add(elem, score=1, pipe=p)

        if pipe is None:
            p.execute()

    def add(self, elem, score=1):
        return bool(self._add(elem, score=score, pipe=self.redis))

    def addnx(self, elem, score=1):
        return bool(self._addnx(elem, score=score, pipe=self.redis))

    def _add(self, elem, score, pipe):
        return pipe.zadd(self.key, score, self._pickle(elem))

    def _addnx(self, elem, score, pipe):
        return self.lua_addnx(keys=[self.key], args=[score, self._pickle(elem)], client=pipe)


    def pop(self, last=False):
        ret = None
        with self.redis.pipeline() as pipe:
            pipe.zrange(self.key, 0, 1, desc=last, withscores=True, score_cast_func=float)
            if last:
                pipe.zremrangebyrank(self.key, -1, -1)
            else:
                pipe.zremrangebyrank(self.key, 0, 0)

            ret = pipe.execute()[0]
            if ret: ret = ret[0]

        return (self._unpickle(ret[0]), ret[1]) if ret else (None, 0)

    def rpop(self):
        return self.pop(last=True)

    def union(*args, **kwargs):
        raise NotImplemented()

    def intersection(*args, **kwargs):
        raise NotImplemented()

    def difference(*args, **kwargs):
        raise NotImplemented()

    def symmetric_difference(*args, **kwargs):
        raise NotImplemented()

    def issubset(*args, **kwargs):
        raise NotImplemented()

    def issuperset(*args, **kwargs):
        raise NotImplemented()


class CountingSet(SortedSet):
    def _add(self, elem, pipe, score=1):
        return pipe.zincrby(self.key, self._pickle(elem), score)

    def __iter__(self):
        return self._data(last=True)

    def __reversed__(self):
        return self._data(last=False)

    def pop(self, last=True):
        return super(CountingSet, self).pop(last)

class PriorityQueue(HoneyBean):

    def __init__(self, *args, **kwargs):
        super(PriorityQueue, self).__init__(*args, **kwargs)

    def __len__(self):
        return self.qsize()

    def qsize(self):
        return self.redis.zcard(self.key)

    def empty(self):
        return True if len(self) > 0 else False

    def _data(self, pipe=None):
        redis = pipe if pipe is not None else self.redis
        return (self._unpickle(v) for v in redis.zrange(self.key, 0, -1))

    def __iter__(self):
        return self._data()

    def __contains__(self, elem):
        rank = self.redis.zscore(self.key, self._pickle(elem))
        return False if rank is None else True

    def put(self, elem, rank=0):
        return bool(self.redis.zadd(self.key, rank, self._pickle(elem)))

    def get(self):
        return self._unpickle(self.redis.zrange(self.key, 0, 1))

    #def _update(self, data, pipe=None):
        #super(PriorityQueue, self)._update(data, pipe)
        #redis = pipe if pipe is not None else self.redis
        #others = [data] + list(others or [])
        #elements = map(self._pickle, data)
        #redis.zadd(self.key, *elements)




# -*- coding: utf-8 -*-
"""
蜂巢的缓存接口，For Cache with Dicts.
"""
from functools import wraps
import os
import redis
import hashlib
import logging

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle

try:
    import simplejson as json
except ImportError:
    import json as json

DEFAULT_EXPIRY = 60 * 60 * 24

class RedisConnect(object):
    def __init__(self, host=None, port=None, db=None):
        self.host = host if host else 'localhost'
        self.port = port if port else 6379
        self.db = db if db else 0

    def _init_redis(self):
        """
        默认初始化redis 实例通过honey.conf的配置
        """
        import yaml
        redis_conf = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'conf', 'honey.conf')
        if os.path.exists(redis_conf):
            with open(redis_conf, 'r') as r:
                redis_conf = yaml.load(r)['redis']
                self.host = redis_conf['host']
                self.port = redis_conf['port']
                self.db = redis_conf['db']

    def connect(self):
        try:
            redis.StrictRedis(host=self.host, port=self.port).ping()
        except redis.ConnectionError as e:
            raise RedisNoConnException("Failed to create connection to redis",
                                       (self.host,
                                        self.port)
            )
        return redis.StrictRedis(host=self.host,
                                 port=self.port,
                                 db=self.db)


class CacheMissException(Exception):
    pass


class ExpiredKeyException(Exception):
    pass


class RedisNoConnException(Exception):
    pass


class DoNotCache(Exception):
    _result = None

    def __init__(self, result):
        super(DoNotCache, self).__init__()
        self._result = result

    @property
    def result(self):
        return self._result

class HoneyCache(object):
    def __init__(self,
                 limit=10000,
                 expire=DEFAULT_EXPIRY,
                 hashkeys=False,
                 host=None,
                 port=None,
                 db=None,
                 namespace="HoneyCache"):

        self.limit = limit
        self.expire = expire
        self.prefix = namespace
        self.host = host
        self.port = port
        self.db = db

        try:
            self.connection = RedisConnect(host=self.host,
                                           port=self.port,
                                           db=self.db).connect()
        except RedisNoConnException, e:
            self.connection = None
            pass

        # 是否应该做一层散列
        self.hashkeys = hashkeys

    def make_key(self, key):
        return "HoneyCache-{0}:{1}".format(self.prefix, key)

    def namespace_key(self, namespace):
        return self.make_key(namespace + ':*')

    def get_set_name(self):
        return "HoneyCache-{0}-keys".format(self.prefix)

    def store(self, key, value, expire=None):
        """
        """
        key = to_unicode(key)
        value = to_unicode(value)
        set_name = self.get_set_name()

        while self.connection.scard(set_name) >= self.limit:
            del_key = self.connection.spop(set_name)
            self.connection.delete(self.make_key(del_key))

        pipe = self.connection.pipeline()
        if expire is None:
            expire = self.expire

        if expire <= 0:
            pipe.set(self.make_key(key), value)
        else:
            pipe.setex(self.make_key(key), expire, value)

        pipe.sadd(set_name, key)
        pipe.execute()


    def expire_all_in_set(self):
        """
        Method expires all keys in the namespace of this object.
        At times there is  a need to invalidate cache in bulk, because a
        single change may result in all data returned by a decorated function
        to be altered.
        Method returns a tuple where first value is total number of keys in
        the set of this object's namespace and second value is a number of
        keys successfully expired.
        :return: int, int
        """
        all_members = self.keys()
        keys  = [self.make_key(k) for k in all_members]

        with self.connection.pipeline() as pipe:
            pipe.delete(*keys)
            pipe.execute()

        return len(self), len(all_members)

    def expire_namespace(self, namespace):
        """
        Method expires all keys in the namespace of this object.
        At times there is  a need to invalidate cache in bulk, because a
        single change may result in all data returned by a decorated function
        to be altered.
        Method returns a tuple where first value is total number of keys in
        the set of this object's namespace and second value is a number of
        keys successfully expired.
        :return: int, int
        """
        namespace = self.namespace_key(namespace)
        all_members = list(self.connection.keys(namespace))
        with self.connection.pipeline() as pipe:
            pipe.delete(*all_members)
            pipe.execute()

        return len(self), len(all_members)

    def isexpired(self, key):
        """
        Method determines whether a given key is already expired. If not expired,
        we expect to get back current ttl for the given key.
        :param key: key being looked-up in Redis
        :return: bool (True) if expired, or int representing current time-to-live (ttl) value
        """
        ttl = self.connection.pttl("HoneyCache-{0}".format(key))
        if ttl == -2: # not exist
            ttl = self.connection.pttl(self.make_key(key))
        elif ttl == -1:
            return True
        if not ttl is None:
            return ttl
        else:
            return self.connection.pttl("{0}:{1}".format(self.prefix, key))

    def store_json(self, key, value, expire=None):
        self.store(key, json.dumps(value), expire)

    def store_pickle(self, key, value, expire=None):
        self.store(key, pickle.dumps(value), expire)

    def get(self, key):
        key = to_unicode(key)
        if key:
            value = self.connection.get(self.make_key(key))
            if value is None:
                if not key in self:
                    raise CacheMissException

                self.connection.srem(self.get_set_name(), key)
                raise ExpiredKeyException
            else:
                return value

    def mget(self, keys):
        """
        Method returns a dict of key/values for found keys.
        :param keys: array of keys to look up in Redis
        :return: dict of found key/values
        """
        if keys:
            cache_keys = [self.make_key(to_unicode(key)) for key in keys]
            values = self.connection.mget(cache_keys)

            if None in values:
                pipe = self.connection.pipeline()
                for cache_key, value in zip(cache_keys, values):
                    if value is None:  # non-existant or expired key
                        pipe.srem(self.get_set_name(), cache_key)
                pipe.execute()

            return {k: v for (k, v) in zip(keys, values) if v is not None}

    def get_json(self, key):
        return json.loads(self.get(key))

    def get_pickle(self, key):
        return pickle.loads(self.get(key))

    def mget_json(self, keys):
        """
        返回一个键/值的dict，每个值将通过标准json 反序列化
        """
        d = self.mget(keys)
        if d:
            for key in d.keys():
                d[key] = json.loads(d[key]) if d[key] else None
            return d

    def invalidate(self, key):
        """
        失效或删除指定key的缓存对象
        """
        key = to_unicode(key)
        pipe = self.connection.pipeline()
        pipe.srem(self.get_set_name(), key)
        pipe.delete(self.make_key(key))
        pipe.execute()

    def __contains__(self, key):
        return self.connection.sismember(self.get_set_name(), key)

    def __iter__(self):
        if not self.connection:
            return iter([])
        return iter(
            ["{0}:{1}".format(self.prefix, x)
                for x in self.connection.smembers(self.get_set_name())
            ])

    def __len__(self):
        return self.connection.scard(self.get_set_name())

    def keys(self):
        return self.connection.smembers(self.get_set_name())


    def flush(self):
        keys = list(self.keys())
        keys.append(self.get_set_name())
        with self.connection.pipeline() as pipe:
            pipe.delete(*keys)
            pipe.execute()

    def flush_namespace(self, space):
        namespace = self.namespace_key(space)
        setname = self.get_set_name()
        keys = list(self.connection.keys(namespace))
        with self.connection.pipeline() as pipe:
            pipe.delete(*keys)
            pipe.srem(setname, *space)
            pipe.execute()

    def get_hash(self, args):
        if self.hashkeys:
            key = hashlib.md5(args).hexdigest()
        else:
            key = pickle.dumps(args)
        return key


def cache_it(limit=10000, expire=DEFAULT_EXPIRY, cache=None,
             use_json=False, namespace=None):

    cache_ = cache
    expire_ = expire
    def decorator(function):
        cache, expire = cache_, expire_
        if cache is None:
            cache = HoneyCache(limit, expire, hashkeys=True, namespace=function.__module__)
        elif expire == DEFAULT_EXPIRY:
            expire = None

        @wraps(function)
        def func(*args, **kwargs):
            if cache.connection is None:
                result = function(*args, **kwargs)
                return result

            serializer = json if use_json else pickle
            fetcher = cache.get_json if use_json else cache.get_pickle
            storer = cache.store_json if use_json else cache.store_pickle

            key = cache.get_hash(serializer.dumps([args, kwargs]))
            cache_key = '{func_name}:{key}'.format(func_name=function.__name__,
                                                   key=key)

            if namespace:
                cache_key = '{namespace}:{key}'.format(namespace=namespace,
                                                       key=cache_key)

            try:
                return fetcher(cache_key)
            except (ExpiredKeyException, CacheMissException) as e:
                # 缓存失效时的逻辑
                pass
            except:
                logging.exception("Unknown redis cache error. Please check your Redis free space.")

            try:
                result = function(*args, **kwargs)
            except DoNotCache as e:
                result = e.result
            else:
                try:
                    storer(cache_key, result, expire)
                except redis.ConnectionError as e:
                    logging.exception(e)

            return result
        return func
    return decorator



def cache_it_json(limit=10000, expire=DEFAULT_EXPIRY, cache=None, namespace=None):
    """
    需要缓存的函数返回必须是可以转换成json结构的类型，比如Dict
    """
    return cache_it(limit=limit, expire=expire, use_json=True,
                    cache=cache, namespace=None)


def to_unicode(obj, encoding='utf-8'):
    if isinstance(obj, basestring):
        if not isinstance(obj, unicode):
            obj = unicode(obj, encoding)
    return obj



# -*- coding: utf-8 -*-
"""
蜂巢的Sets 结构接口，集合及排序集合.
"""

import itertools
import collections
from abc import ABCMeta, abstractmethod
from .base import HoneyBean, same_types

class SetOperation(object):

    __metaclass__ = ABCMeta

    def __init__(self, s, update=False, flipped=False, return_cls=None):
        """
        """
        assert not (update and flipped)

        self.s = s
        self.update = update
        self.flipped = flipped
        self.return_cls = None if (update or flipped) else return_cls

    def _are_set_instances(self, *others):
        test = lambda other: isinstance(other, Set)
        if len(others) == 1:
            return test(others[0])
        return all(map(test, others))

    def _to_set(self, c, pipe=None):
        if isinstance(c, HoneyBean):
            return set(c._data(pipe=pipe))
        return set(c)

    def _op(self, key, return_cls, others):
        if self.flipped:
            assert len(others) == 1
            left = others[0]
            right = [self.s]
        else:
            left = self.s
            right = others

        def trans(pipe):
            # retrieve
            data = self._to_set(left, pipe=pipe)
            other_sets = [self._to_set(o, pipe=pipe) for o in right]

            # operation
            elements = self.op(data, other_sets)
            pipe.multi()

            # store within the transaction
            return self.s._create_new(elements, key=key, cls=return_cls,
                                      pipe=pipe)
        return self.s._transaction(trans, key)

    @abstractmethod
    def op(self, s, other_sets):
        pass

    def _redisop(self, key, return_cls, other_keys):
        if self.flipped:
            assert len(other_keys) == 1
            left = other_keys[0]
            right = [self.s.key]
        else:
            left = self.s.key
            right = other_keys

        def trans(pipe):
            elements = self.redisop(pipe, left, right)
            pipe.multi()

            return self.s._create_new(elements, key=key, cls=return_cls,
                                      pipe=pipe)
        return self.s._transaction(trans, key, *other_keys)

    @abstractmethod
    def redisop(self, pipe, key, other_keys):
        pass

    def _redisopstore(self, key, return_cls, other_keys):
        if self.flipped:
            assert len(other_keys) == 1
            left = other_keys[0]
            right = [self.s.key]
        else:
            left = self.s.key
            right = other_keys

        def trans(pipe):
            new = self.s._create_new(key=key, cls=return_cls)
            self.redisopstore(pipe, new.key, left, right)
            return new
        return self.s._transaction(trans, key, *other_keys)

    @abstractmethod
    def redisopstore(self, pipe, new_key, key, other_keys):
        pass

    def __call__(self, *others):
        if self.flipped:
            assert len(others) == 1
            return_cls = others[0].__class__
        elif self.update:
            return_cls = self.s.__class__
        else:
            return_cls = self.return_cls or Set

        new_key = self.s.key if self.update else None

        if self._are_set_instances(*others):
            other_keys = [other.key for other in others]

            if issubclass(return_cls, self.s.__class__):
                return self._redisopstore(new_key, return_cls, other_keys)
            else:
                return self._redisop(new_key, return_cls, other_keys)

        return self._op(new_key, return_cls, others)


class SetDifference(SetOperation):

    def op(self, s, other_sets):
        if self.update:
            s.difference_update(*other_sets)
            return s
        return s.difference(*other_sets)

    def redisop(self, pipe, key, other_keys):
        return pipe.sdiff(key, *other_keys)

    def redisopstore(self, pipe, new_key, key, other_keys):
        pipe.multi()
        pipe.sdiffstore(new_key, key, *other_keys)


class SetIntersection(SetOperation):

    def op(self, s, other_sets):
        if self.update:
            s.intersection_update(*other_sets)
            return s
        return s.intersection(*other_sets)

    def redisop(self, pipe, key, other_keys):
        return pipe.sinter(key, *other_keys)

    def redisopstore(self, pipe, new_key, key, other_keys):
        pipe.multi()
        pipe.sinterstore(new_key, key, *other_keys)


class SetUnion(SetOperation):

    def op(self, s, other_sets):
        if self.update:
            s.update(*other_sets)
            return s
        return s.union(*other_sets)

    def redisop(self, pipe, key, other_keys):
        return pipe.suninon(key, *other_keys)

    def redisopstore(self, pipe, new_key, key, other_keys):
        pipe.multi()
        pipe.sunionstore(new_key, key, *other_keys)


class SetSymmetricDifference(SetOperation):

    def op(self, s, other_sets):
        if self.update:
            s.symmetric_difference_update(*other_sets)
            return s
        return s.symmetric_difference(*other_sets)

    def _simulate_redisop(self, pipe, key, other_key):
        diff1 = pipe.sdiff(key, other_key)
        diff2 = pipe.sdiff(other_key, key)
        return diff1 | diff2

    def redisop(self, pipe, key, other_keys):
        other_key = other_keys[0]
        elements = self._simulate_redisop(pipe, key, other_key)
        return map(self.s._unpickle, elements)

    def redisopstore(self, pipe, new_key, key, other_keys):
        other_key = other_keys[0]
        elements = self._simulate_redisop(pipe, key, other_key)
        pipe.multi()
        pipe.delete(new_key)
        pipe.sadd(new_key, *elements)


class Set(HoneyBean, collections.MutableSet):

    _same_types = (collections.Set,)

    def __init__(self, *args, **kwargs):
        super(Set, self).__init__(*args, **kwargs)

    def __len__(self):
        return self.redis.scard(self.key)

    def _data(self, pipe=None):
        redis = pipe if pipe is not None else self.redis
        return (self._unpickle(v) for v in redis.smembers(self.key))

    def __iter__(self):
        return self._data()

    def __contains__(self, elem):
        return self.redis.sismember(self.key, self._pickle(elem))

    def add(self, elem):
        return bool(self.redis.sadd(self.key, self._pickle(elem)))

    def discard(self, elem):
        self.redis.srem(self.key, self._pickle(elem))

    def remove(self, elem):
        removed_count = self.redis.srem(self.key, self._pickle(elem))
        if not removed_count:
            raise KeyError(elem)

    def pop(self):
        with self.redis.pipeline() as pipe:
            pipe.scard(self.key)
            pipe.spop(self.key)
            size, elem = pipe.execute()

        if not size:
            raise KeyError
        return self._unpickle(elem)

    def random_sample(self, k=1):
        if k < 1:
            return []
        if k == 1:
            elements = [self.redis.srandmember(self.key)]
        else:
            elements = self.redis.srandmember(self.key, k)
        return map(self._unpickle, elements)

    def difference(self, *others, **kwargs):
        op = SetDifference(self, return_cls=kwargs.get('return_cls'))
        return op(*others)

    @same_types
    def __sub__(self, other):
        return self.difference(other)

    @same_types
    def __rsub__(self, other):
        op = SetDifference(self, flipped=True)
        return op(other)

    def difference_update(self, *others):
        op = SetDifference(self, update=True)
        op(*others)

    @same_types
    def __isub__(self, other):
        op = SetDifference(self, update=True)
        return op(other)

    def intersection(self, *others, **kwargs):
        op = SetIntersection(self, return_cls=kwargs.get('return_cls'))
        return op(*others)

    @same_types
    def __and__(self, other):
        return self.intersection(other)

    @same_types
    def __rand__(self, other):
        op = SetIntersection(self, flipped=True)
        return op(other)

    def intersection_update(self, *others):
        op = SetIntersection(self, update=True)
        op(*others)

    @same_types
    def __iand__(self, other):
        op = SetIntersection(self, update=True)
        return op(other)

    def union(self, *others, **kwargs):
        op = SetUnion(self, return_cls=kwargs.get('return_cls'))
        return op(*others)

    @same_types
    def __or__(self, other):
        return self.union(other)

    @same_types
    def __ror__(self, other):
        return self.union(other, return_cls=other.__class__)

    def _update(self, data, others=None, pipe=None):
        super(Set, self)._update(data, pipe)
        redis = pipe if pipe is not None else self.redis

        others = [data] + list(others or [])
        elements = map(self._pickle, frozenset(itertools.chain(*others)))

        redis.sadd(self.key, *elements)

    def update(self, *others):
        op = SetUnion(self, update=True)
        op(*others)

    @same_types
    def __ior__(self, other):
        op = SetUnion(self, update=True)
        return op(other)

    def symmetric_difference(self, other, **kwargs):
        op = SetSymmetricDifference(self, return_cls=kwargs.get('return_cls'))
        return op(other)

    @same_types
    def __xor__(self, other):
        return self.symmetric_difference(other)

    @same_types
    def __rxor__(self, other):
        return self.symmetric_difference(other, return_cls=other.__class__)

    def symmetric_difference_update(self, other):
        op = SetSymmetricDifference(self, update=True)
        op(other)

    @same_types
    def __ixor__(self, other):
        op = SetSymmetricDifference(self, update=True)
        return op(other)

    def __eq__(self, other):
        if not isinstance(other, collections.Set):
            return NotImplemented
        if isinstance(other, Set):
            with self.redis.pipeline() as pipe:
                pipe.smembers(self.key)
                pipe.smembers(other.key)
                members1, members2 = pipe.execute()
            return members1 == members2
        return frozenset(self) == frozenset(other)

    def __le__(self, other):
        if not isinstance(other, collections.Set):
            return NotImplemented
        return self.issubset(other)

    def __lt__(self, other):
        if not isinstance(other, collections.Set):
            return NotImplemented
        if isinstance(other, Set):
            with self.redis.pipeline() as pipe:
                pipe.smembers(self.key)
                pipe.sinter(self.key, other.key)
                pipe.scard(other.key)
                members, inters, other_size = pipe.execute()
            return (members == inters and len(members) != other_size)
        return frozenset(self) < frozenset(other)

    def issubset(self, other):
        if isinstance(other, Set):
            with self.redis.pipeline() as pipe:
                pipe.smembers(self.key)
                pipe.sinter(self.key, other.key)
                members, inters = pipe.execute()
            return members == inters
        return frozenset(self) <= frozenset(other)

    def issuperset(self, other):
        if isinstance(other, collections.Set):
            return other <= self
        else:
            return frozenset(other) <= self

    def _repr_data(self, data):
        list_repr = repr(list(data))
        set_repr = '{' + list_repr[1:-1] + '}'
        return set_repr



# -*- coding: utf-8 -*-
__title__ = 'honeycomb'
__version__ = '0.1.0'
__author__ = 'tony lee'

from .base import HoneyBean
from .hashs import Dict, Counter
from .cache import HoneyCache, cache_it, cache_it_json
from .sortedset import SortedSet, PriorityQueue
from .sets import Set
from .lists import List



#-*- encoding: UTF-8 -*-
from setuptools import setup

VERSION = '0.1.9'

long_description = '''
使用方式

from honeycomb import Dict, Counter, HoneyCache, cache_it, cache_it_json, SortedSet, List

if __name__ == '__main__':
    import time
    #排序set
    s = SortedSet(key='testing')
    for i in range(10):
        s.add(i, int(time.time()))

    #升序
    for i in s.chunk(10, 5, last=True):
        print i

    c = Counter(key='like')
    c.incr('shop_item_1')
    c.keys() # or c.items()

    d = Dcit(key='like')
    d['user_id'] = {'click': '1'}

    d.items() # or d.popitem()

    @cache_it(limit=100, expire=60 * 60, namespace="user_like")
    def testCacheBean(n):
        print 'Cache for namespace: ', n
        return [i for i in range(n)]

    testCacheBean(10)

    # list

    ls = List(key='test_ls')
    ls.append(1)
    print ls[0]
'''

setup(
      name='honeycomb',
      version=VERSION,
      description="a tiny and smart redis data structure wrapper of honey project based on Python",
      long_description=long_description,
      classifiers=[],
      keywords='python redis structure wrapper middleware',
      author='tony lee',
      author_email='liwei@qfpay.com',
      url='',
      license='MIT',
      packages=['honeycomb'],
      include_package_data=True,
      zip_safe=True,
      install_requires=[
        'redis',
      ],
)



# -*- coding: utf-8 -*-
import time
from honeycomb import Dict, Counter, HoneyCache, cache_it, cache_it_json, SortedSet, PriorityQueue

def testDict():
    c = Dict(key='cache')
    val = [i for i in range(10)]
    c['self'] = val

    assert c.get('self') == val and isinstance(c.get('self'), list)
    assert 7 in c['self']

    c['super'] = dict(zip([i for i in range(10)], [j for j in range(10, 20)]))
    #print c.keys()
    #print c.values()
    for k,v in c.iteritems():
        print k, v
    print c.getmany(['self', 'super'])


def testCounter():
    c = Counter(key='shop')
    c.incr('shop_item_1')
    c.incr('test1')
    c.incr('test1')
    c.incr('test2')
    c.incr('test2')
    c.decr('test2')
    #print c.most_common(n=10)
    other = Counter(key='counter1')
    other.incr('test3')

    #print c & other
    #print c | other
    c.subtract(other)
    c.update(other)
    print c.keys()

@cache_it_json(limit=1000, expire=60 * 60 * 1)
def testCacheJson(n):
    time.sleep(2)
    print 'Cache param for : ',n
    return [i for i in range(n)]

@cache_it(limit=100, expire=60 * 60, namespace="user")
def testCacheBean(n):
    time.sleep(1)
    print 'Cache for namespace: ', n
    return [i for i in range(n)]


def testHoneyCache():
    c = HoneyCache(10)  # 缓存的item最大限制10个
    c.store("hello", "world")
    assert c.get("hello") == 'world'
    print "hello" in c
    assert len(c) == 1
    print c.keys()
    print c.flush() # 清掉缓存
    print c.keys()


def test_sortedset():
    import time
    s = SortedSet(key='testing')
    #for i in range(10):
    #    s.add(i, int(time.time()))
    #s2 = SortedSet(key='testing2')
    #for i in range(5,15):
    #    s.add(i, int(time.time()))

    for i in s.chunk(10, 5, last=True):
        print i
    ##print s.key
    #s.discard(2)

def test_queue():
    q = PriorityQueue(key='test_queue')
    #print q.qsize()
    #print q.put("timeline")
    print q.get()
    #q.clear()


if __name__ == '__main__' :
    test_queue()
    #test_sortedset()
    #testCounter()
    #testDict()
    #print testCacheJson(5)
    #print testHoneyCache()
    #print testCacheBean(6)
